@@ -1,6 +1,5 @@
-// FlowLayout.java - Grid-based layout engine
-
-/* Copyright (C) 1999, 2000, 2001, 2002  Free Software Foundation
+/* FlowLayout.java -- Grid-based layout engine
+   Copyright (C) 1999, 2000, 2001, 2002, 2004  Free Software Foundation
 
 This file is part of GNU Classpath.
 
@@ -47,7 +46,7 @@ import java.io.Serializable;
  * supports horizontal and vertical gaps.  These are used for spacing
  * between components.
  *
- * @author Tom Tromey <tromey@redhat.com>
+ * @author Tom Tromey (tromey@redhat.com)
  * @author Aaron M. Renn (arenn@urbanophile.com)
  */
 public class FlowLayout implements LayoutManager, Serializable
@@ -214,7 +213,8 @@ public class FlowLayout implements LayoutManager, Serializable
 		if (comps[k].visible)
 		  {
 		    Dimension c = comps[k].getPreferredSize ();
-		    comps[k].setBounds (x, y, c.width, new_h);
+		    comps[k].setBounds (x, y + (new_h - c.height) / 2,
+					c.width, c.height);
 		    x += c.width + hgap;
 		  }
 	      }
