@@ -1,5 +1,5 @@
 /* KeyboardFocusManager.java -- manage component focusing via the keyboard
-   Copyright (C) 2002 Free Software Foundation
+   Copyright (C) 2002, 2004  Free Software Foundation
 
 This file is part of GNU Classpath.
 
@@ -38,8 +38,8 @@ exception statement from your version. */
 
 package java.awt;
 
-import java.awt.event.KeyEvent;
 import java.awt.event.FocusEvent;
+import java.awt.event.KeyEvent;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.beans.PropertyVetoException;
@@ -262,6 +262,10 @@ public abstract class KeyboardFocusManager
   public static KeyboardFocusManager getCurrentKeyboardFocusManager ()
   {
     ThreadGroup currentGroup = Thread.currentThread ().getThreadGroup ();
+
+    if (currentKeyboardFocusManagers.get (currentGroup) == null)
+      setCurrentKeyboardFocusManager (null);
+
     return (KeyboardFocusManager) currentKeyboardFocusManagers.get (currentGroup);
   }
 
@@ -1100,7 +1104,7 @@ public abstract class KeyboardFocusManager
   {
     synchronized (e)
       {
-    e.setSource (target);
+        e.setSource (target);
         target.dispatchEvent (e);
       }
   }
