@@ -1,5 +1,5 @@
 /* MemoryImageSource.java -- Java class for providing image data 
-   Copyright (C) 1999 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2004  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,9 +38,8 @@ exception statement from your version. */
 
 package java.awt.image;
 
-import java.awt.Image;
-import java.util.Enumeration;
 import java.util.Hashtable;
+import java.util.Vector;
 
 public class MemoryImageSource implements ImageProducer 
 {
@@ -49,7 +48,8 @@ public class MemoryImageSource implements ImageProducer
     private int pixeli[], width, height, offset, scansize;
     private byte pixelb[];
     private ColorModel cm;
-    private Hashtable props, consumers = new Hashtable();
+    private Hashtable props = new Hashtable();
+    private Vector consumers = new Vector();
 
     /**
        Constructs an ImageProducer from memory
@@ -126,10 +126,10 @@ public class MemoryImageSource implements ImageProducer
      * <code>ImageProducer</code>.  
      */
     public synchronized void addConsumer(ImageConsumer ic) {
-	if (consumers.containsKey(ic))
+	if (consumers.contains(ic))
 	    return;
 
-	consumers.put(ic, ic);
+	consumers.addElement(ic);
     }
 
     /**
@@ -137,7 +137,7 @@ public class MemoryImageSource implements ImageProducer
      * already registered with this <code>ImageProducer</code>.  
      */
     public synchronized boolean isConsumer(ImageConsumer ic) {
-	if (consumers.containsKey(ic))
+	if (consumers.contains(ic))
 	    return true;
 	return false;
     }
@@ -147,7 +147,7 @@ public class MemoryImageSource implements ImageProducer
      * registered consumers for this <code>ImageProducer</code>.  
      */
     public synchronized void removeConsumer(ImageConsumer ic) {
-	consumers.remove(ic);
+	consumers.removeElement(ic);
     }
 
     /**
@@ -157,16 +157,16 @@ public class MemoryImageSource implements ImageProducer
      * registered consumers.  
      */
     public void startProduction(ImageConsumer ic) {
-	if (!(consumers.containsKey(ic))) {
-	    consumers.put(ic, ic);
+	if (!(consumers.contains(ic))) {
+	    consumers.addElement(ic);
 	}        
-	Enumeration e = consumers.elements();
-	for( ; e.hasMoreElements(); ) {
-		ic = (ImageConsumer)e.nextElement();
-		sendPicture( ic );
-		ic.imageComplete( ImageConsumer.SINGLEFRAME );
-	    }	
 
+	Vector list = (Vector) consumers.clone();
+	for(int i = 0; i < list.size(); i++) {
+	    ic = (ImageConsumer) list.elementAt(i);
+	    sendPicture( ic );
+	    ic.imageComplete( ImageConsumer.STATICIMAGEDONE );
+	}	
     }
 
     /**
@@ -210,12 +210,12 @@ public class MemoryImageSource implements ImageProducer
     {
 	if( animated == true ) {
 		ImageConsumer ic;
-		Enumeration e = consumers.elements();
-		for( ; e.hasMoreElements(); ) {
-			ic = (ImageConsumer)e.nextElement();
+		Vector list = (Vector) consumers.clone();
+		for(int i = 0; i < list.size(); i++) {
+			ic = (ImageConsumer) list.elementAt(i);
 			sendPicture( ic );
 			ic.imageComplete( ImageConsumer.SINGLEFRAME );
-		    }	
+		}	
 	    }
     }
 
@@ -227,6 +227,7 @@ public class MemoryImageSource implements ImageProducer
 	    ic.setProperties( props );
 	}
 	ic.setDimensions(width, height);
+	ic.setColorModel(cm);
 	if( pixeli != null ) {
 	    ic.setPixels( 0, 0, width, height, cm, pixeli, offset, scansize );
 	} else {
@@ -249,17 +250,23 @@ public class MemoryImageSource implements ImageProducer
 		    newPixels();
 		} else {
 		    ImageConsumer ic;
-		    Enumeration e = consumers.elements();
-		    for( ; e.hasMoreElements(); ) {
-			    ic = (ImageConsumer)e.nextElement();
+		    Vector list = (Vector) consumers.clone();
+		    for(int i = 0; i < list.size(); i++) {
+			    ic = (ImageConsumer) list.elementAt(i);
 			    ic.setHints( ImageConsumer.TOPDOWNLEFTRIGHT );
 			    if( props != null ) {
 				ic.setProperties( props );
 			    }
 			    if( pixeli != null ) {
-				ic.setPixels( 0, 0, width, height, cm, pixeli, offset, scansize );
+				int[] pixelbuf = new int[w * h];
+				for (int row = y; row < h; row++)
+				    System.arraycopy(pixeli, row * scansize + x + offset, pixelbuf, row * w, w);
+				ic.setPixels( x, y, w, h, cm, pixelbuf, 0, w );
 			    } else {
-				ic.setPixels( 0, 0, width, height, cm, pixelb, offset, scansize );
+				byte[] pixelbuf = new byte[w * h];
+				for (int row = y; row < h; row++)
+				    System.arraycopy(pixelb, row * scansize + x + offset, pixelbuf, row * w, w);
+				ic.setPixels( x, y, w, h, cm, pixelbuf, 0, w );
 			    }
 			    ic.imageComplete( ImageConsumer.SINGLEFRAME );
 		    }
@@ -288,17 +295,23 @@ public class MemoryImageSource implements ImageProducer
 		    newPixels();
 		} else {
 		    ImageConsumer ic;
-		    Enumeration e = consumers.elements();
-		    for( ; e.hasMoreElements(); ) {
-			    ic = (ImageConsumer)e.nextElement();
+		    Vector list = (Vector) consumers.clone();
+		    for(int i = 0; i < list.size(); i++) {
+			    ic = (ImageConsumer) list.elementAt(i);
 			    ic.setHints( ImageConsumer.TOPDOWNLEFTRIGHT );
 			    if( props != null ) {
 				ic.setProperties( props );
 			    }
 			    if( pixeli != null ) {
-				ic.setPixels( 0, 0, width, height, cm, pixeli, offset, scansize );
+				int[] pixelbuf = new int[w * h];
+				for (int row = y; row < h; row++)
+				    System.arraycopy(pixeli, row * scansize + x + offset, pixelbuf, row * w, w);
+				ic.setPixels( x, y, w, h, cm, pixelbuf, 0, w );
 			    } else {
-				ic.setPixels( 0, 0, width, height, cm, pixelb, offset, scansize );
+				byte[] pixelbuf = new byte[w * h];
+				for (int row = y; row < h; row++)
+				    System.arraycopy(pixelb, row * scansize + x + offset, pixelbuf, row * w, w);
+				ic.setPixels( x, y, w, h, cm, pixelbuf, 0, w );
 			    }
 			    if( framenotify == true )
 				ic.imageComplete( ImageConsumer.SINGLEFRAME );
@@ -313,9 +326,14 @@ public class MemoryImageSource implements ImageProducer
 				       int scansize)
 
     {
+	pixeli = null;
+	pixelb = newpix;
+	cm = newmodel;
+	this.offset = offset;
+	this.scansize = scansize;
 	if( animated == true )
 	    {
-		//FIXME
+		newPixels();
 	    }
     }
 
@@ -325,9 +343,14 @@ public class MemoryImageSource implements ImageProducer
 				       int scansize)
 
     {
+	pixelb = null;
+	pixeli = newpix;
+	cm = newmodel;
+	this.offset = offset;
+	this.scansize = scansize;
 	if( animated == true )
 	    {
-		//FIXME
+		newPixels();
 	    }
     }
 
