@@ -1,5 +1,5 @@
 /* PrintStream.java -- OutputStream for printing output
-   Copyright (C) 1998, 1999, 2001, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2001, 2003, 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -80,6 +80,44 @@ public class PrintStream extends FilterOutputStream
    */
   private boolean closed;
 
+  /**
+   * This class exists to forward the write calls from the PrintWriter back
+   * to us. This is required to make subclassing of PrintStream work
+   * correctly.
+   */
+  private class ForwardStream extends OutputStream
+  {
+    // This is package-private to avoid a trampoline constructor.
+    ForwardStream ()
+    {
+    }
+
+    public void close () throws IOException
+    {
+      out.close ();
+    }
+
+    public void flush () throws IOException
+    {
+      out.flush ();
+    }
+
+    public void write (byte[] b) throws IOException
+    {
+	PrintStream.this.write (b);
+    }
+
+    public void write (byte[] b, int off, int len) throws IOException
+    {
+	PrintStream.this.write (b, off, len);
+    }
+
+    public void write (int b) throws IOException
+    {
+	PrintStream.this.write (b);
+    }
+  }
+
   /**
    * This method intializes a new <code>PrintStream</code> object to write
    * to the specified output sink.
@@ -108,7 +146,10 @@ public class PrintStream extends FilterOutputStream
   {
     super (out);
 
-    pw = new PrintWriter (out, auto_flush);
+    // FIXME Instead of using PrintWriter and ForwardStream we
+    // should inline the character conversion (see libgcj's version
+    // of this class)
+    pw = new PrintWriter (new ForwardStream (), auto_flush);
     this.auto_flush = auto_flush;
   }
 
@@ -132,7 +173,12 @@ public class PrintStream extends FilterOutputStream
   {
     super (out);
 
-    pw = new PrintWriter (new OutputStreamWriter (out, encoding), auto_flush);
+    // FIXME Instead of using PrintWriter and ForwardStream we
+    // should inline the character conversion (see libgcj's version
+    // of this class)
+    pw = new PrintWriter (
+	    new OutputStreamWriter (
+		new ForwardStream (), encoding), auto_flush);
     this.auto_flush = auto_flush;
   }
 
@@ -435,6 +481,10 @@ public class PrintStream extends FilterOutputStream
         if (auto_flush && (oneByte == '\n'))
           flush ();
       }
+    catch (InterruptedIOException iioe)
+      {
+	Thread.currentThread ().interrupt ();
+      }
     catch (IOException e)
       {
         setError ();
@@ -462,10 +512,13 @@ public class PrintStream extends FilterOutputStream
         if (auto_flush)
           flush ();
       }
+    catch (InterruptedIOException iioe)
+      {
+	Thread.currentThread ().interrupt ();
+      }
     catch (IOException e)
       {
         setError ();
       }
   }
 } // class PrintStream
-
