@@ -1,5 +1,5 @@
 /* ClassLoader.java -- responsible for loading classes into the VM
-   Copyright (C) 1998, 1999, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -41,16 +41,21 @@ package java.lang;
 import gnu.java.util.DoubleEnumeration;
 import gnu.java.util.EmptyEnumeration;
 
+import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
+import java.lang.reflect.Constructor;
 import java.net.URL;
+import java.net.URLClassLoader;
 import java.security.CodeSource;
 import java.security.PermissionCollection;
 import java.security.Policy;
 import java.security.ProtectionDomain;
+import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.StringTokenizer;
 
 /**
  * The ClassLoader is a way of customizing the way Java gets its classes
@@ -79,8 +84,8 @@ import java.util.Map;
  * <p>The bootstrap classloader in GNU Classpath is implemented as a couple of
  * static (native) methods on the package private class
  * <code>java.lang.VMClassLoader</code>, the system classloader is an
- * instance of <code>gnu.java.lang.SystemClassLoader</code>
- * (which is a subclass of <code>java.net.URLClassLoader</code>).
+ * anonymous inner class of ClassLoader and a subclass of
+ * <code>java.net.URLClassLoader</code>.
  *
  * <p>Users of a <code>ClassLoader</code> will normally just use the methods
  * <ul>
@@ -122,13 +127,13 @@ public abstract class ClassLoader
    * is not private in order to allow native code (and trusted subclasses)
    * access to this field.
    */
-  final Map loadedClasses = new HashMap();
+  final HashMap loadedClasses = new HashMap();
 
   /**
    * All packages defined by this classloader. It is not private in order to
    * allow native code (and trusted subclasses) access to this field.
    */
-  final Map definedPackages = new HashMap();
+  final HashMap definedPackages = new HashMap();
 
   /**
    * The classloader that is consulted before this classloader.
@@ -145,17 +150,6 @@ public abstract class ClassLoader
    */
   private final boolean initialized;
 
-  /**
-   * System/Application classloader: defaults to an instance of
-   * gnu.java.lang.SystemClassLoader, unless the first invocation of
-   * getSystemClassLoader loads another class loader because of the
-   * java.system.class.loader property. The initialization of this field
-   * is somewhat circular - getSystemClassLoader() checks whether this
-   * field is null in order to bypass a security check.
-   */
-  static final ClassLoader systemClassLoader =
-    VMClassLoader.getSystemClassLoader();
-
   /**
    * The default protection domain, used when defining a class with a null
    * paramter for the domain.
@@ -217,7 +211,7 @@ public abstract class ClassLoader
    */
   protected ClassLoader() throws SecurityException
   {
-    this(systemClassLoader);
+    this(System.systemClassLoader);
   }
 
   /**
@@ -466,7 +460,7 @@ public abstract class ClassLoader
   protected final Class findSystemClass(String name)
     throws ClassNotFoundException
   {
-    return Class.forName(name, false, systemClassLoader);
+    return Class.forName(name, false, System.systemClassLoader);
   }
 
   /**
@@ -628,7 +622,7 @@ public abstract class ClassLoader
    */
   public static final URL getSystemResource(String name)
   {
-    return systemClassLoader.getResource(name);
+    return System.systemClassLoader.getResource(name);
   }
 
   /**
@@ -644,7 +638,7 @@ public abstract class ClassLoader
    */
   public static Enumeration getSystemResources(String name) throws IOException
   {
-    return systemClassLoader.getResources(name);
+    return System.systemClassLoader.getResources(name);
   }
 
   /**
@@ -699,18 +693,18 @@ public abstract class ClassLoader
 
   /**
    * Returns the system classloader. The system classloader (also called
-   * the application classloader) is the classloader that was used to
+   * the application classloader) is the classloader that is used to
    * load the application classes on the classpath (given by the system
    * property <code>java.class.path</code>. This is set as the context
    * class loader for a thread. The system property
    * <code>java.system.class.loader</code>, if defined, is taken to be the
    * name of the class to use as the system class loader, which must have
-   * a public constructor which takes a ClassLoader as a parent; otherwise this
-   * uses gnu.java.lang.SystemClassLoader.
+   * a public constructor which takes a ClassLoader as a parent. The parent
+   * class loader passed in the constructor is the default system class
+   * loader.
    *
    * <p>Note that this is different from the bootstrap classloader that
-   * actually loads all the real "system" classes (the bootstrap classloader
-   * is the parent of the returned system classloader).
+   * actually loads all the real "system" classes.
    *
    * <p>A security check will be performed for
    * <code>RuntimePermission("getClassLoader")</code> if the calling class
@@ -730,11 +724,11 @@ public abstract class ClassLoader
       {
 	Class c = VMSecurityManager.getClassContext()[1];
 	ClassLoader cl = c.getClassLoader();
-	if (cl != null && cl != systemClassLoader)
+	if (cl != null && cl != System.systemClassLoader)
 	  sm.checkPermission(new RuntimePermission("getClassLoader"));
       }
 
-    return systemClassLoader;
+    return System.systemClassLoader;
   }
 
   /**
@@ -950,4 +944,121 @@ public abstract class ClassLoader
       }
     return false;
   }
+
+  private static URL[] getExtClassLoaderUrls()
+  {
+    String classpath = getSystemProperty("java.ext.dirs", "");
+    StringTokenizer tok = new StringTokenizer(classpath, File.pathSeparator);
+    ArrayList list = new ArrayList();
+    while (tok.hasMoreTokens())
+      {
+	try
+	  {
+	    File f = new File(tok.nextToken());
+	    File[] files = f.listFiles();
+	    for (int i = 0; i < files.length; i++)
+	      {
+		list.add(files[i].toURL());
+	      }
+	  }
+	catch(Exception x)
+	  {
+	  }
+      }
+    URL[] urls = new URL[list.size()];
+    list.toArray(urls);
+    return urls;
+  }
+
+  private static void addFileURL(ArrayList list, String file)
+  {
+    try
+      {
+	list.add(new File(file).toURL());
+      }
+    catch(java.net.MalformedURLException x)
+      {
+      }
+  }
+
+  private static URL[] getSystemClassLoaderUrls()
+  {
+    String classpath = getSystemProperty("java.class.path", ".");
+    StringTokenizer tok = new StringTokenizer(classpath, File.pathSeparator, true);
+    ArrayList list = new ArrayList();
+    while (tok.hasMoreTokens())
+      {
+	String s = tok.nextToken();
+	if (s.equals(File.pathSeparator))
+	    addFileURL(list, ".");
+	else
+	  {
+	    addFileURL(list, s);
+	    if (tok.hasMoreTokens())
+	      {
+		// Skip the separator.
+		tok.nextToken();
+		// If the classpath ended with a separator,
+		// append the current directory.
+		if (!tok.hasMoreTokens())
+		    addFileURL(list, ".");
+	      }
+	  }
+      }
+    URL[] urls = new URL[list.size()];
+    list.toArray(urls);
+    return urls;
+  }
+
+  static ClassLoader defaultGetSystemClassLoader()
+  {
+    ClassLoader extClassLoader =
+	new URLClassLoader(getExtClassLoaderUrls(), null);
+    ClassLoader systemClassLoader =
+	new URLClassLoader(getSystemClassLoaderUrls(), extClassLoader)
+	{
+	    protected synchronized Class loadClass(String name,
+		boolean resolve)
+		throws ClassNotFoundException
+	    {
+		SecurityManager sm = Runtime.securityManager;
+		if (sm != null)
+		{
+		    int lastDot = name.lastIndexOf('.');
+		    if (lastDot != -1)
+			sm.checkPackageAccess(name.substring(0, lastDot));
+		}
+		return super.loadClass(name, resolve);
+	    }
+	};
+    String loader = getSystemProperty("java.system.class.loader", null);
+    if (loader == null)
+      {
+	return systemClassLoader;
+      }
+    try
+      {
+	Constructor c = Class.forName(loader, false, systemClassLoader)
+	    .getConstructor(new Class[] { ClassLoader.class });
+	return (ClassLoader)c.newInstance(new Object[] { systemClassLoader });
+      }
+    catch (Exception e)
+      {
+	System.err.println("Requested system classloader " + loader + " failed.");
+	throw (Error)
+	    new Error("Requested system classloader " + loader + " failed.")
+		.initCause(e);
+      }
+  }
+
+  static String getSystemProperty(String name, String defaultValue)
+  {
+    // access properties directly to bypass security
+    String val = System.properties.getProperty(name);
+    if (val == null)
+      {
+	val = defaultValue;
+      }
+    return val;
+  }
 }
