@@ -51,7 +51,7 @@ import java.util.PropertyPermission;
  * general environment.  As such, all methods are static.
  *
  * @author John Keiser
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @since 1.0
  * @status still missing 1.4 functionality
  */
@@ -60,6 +60,59 @@ public final class System
   // WARNING: System is a CORE class in the bootstrap cycle. See the comments
   // in vm/reference/java/lang/Runtime for implications of this fact.
 
+  /**
+   * The System Class Loader (a.k.a. Application Class Loader). The one
+   * returned by ClassLoader.getSystemClassLoader. It lives here to prevent
+   * a circular initialization dependency between System and ClassLoader.
+   *
+   * We can't make it a blank final, since initSystemClassLoader is a
+   * sub-function.
+   */
+  static ClassLoader systemClassLoader;
+
+  /**
+   * Stores the current system properties. This can be modified by
+   * {@link #setProperties(Properties)}, but will never be null, because
+   * setProperties(null) sucks in the default properties.
+   */
+  static Properties properties;
+
+  /**
+   * The standard InputStream. This is assigned at startup and starts its
+   * life perfectly valid. Although it is marked final, you can change it
+   * using {@link #setIn(InputStream)} through some hefty VM magic.
+   *
+   * <p>This corresponds to the C stdin and C++ cin variables, which
+   * typically input from the keyboard, but may be used to pipe input from
+   * other processes or files.  That should all be transparent to you,
+   * however.
+   */
+  public static final InputStream in;
+
+  /**
+   * The standard output PrintStream.  This is assigned at startup and
+   * starts its life perfectly valid. Although it is marked final, you can
+   * change it using {@link #setOut(PrintStream)} through some hefty VM magic.
+   *
+   * <p>This corresponds to the C stdout and C++ cout variables, which
+   * typically output normal messages to the screen, but may be used to pipe
+   * output to other processes or files.  That should all be transparent to
+   * you, however.
+   */
+  public static final PrintStream out;
+
+  /**
+   * The standard output PrintStream.  This is assigned at startup and
+   * starts its life perfectly valid. Although it is marked final, you can
+   * change it using {@link #setErr(PrintStream)} through some hefty VM magic.
+   *
+   * <p>This corresponds to the C stderr and C++ cerr variables, which
+   * typically output error messages to the screen, but may be used to pipe
+   * output to other processes or files.  That should all be transparent to
+   * you, however.
+   */
+  public static final PrintStream err;
+
   /**
    * Add to the default properties. The field is stored in Runtime, because
    * of the bootstrap sequence; but this adds several useful properties to
@@ -69,12 +122,32 @@ public final class System
    */
   static
   {
+    if (! Configuration.JAVA_LANG_SYSTEM_EXPLICIT_INITIALIZATION) {
+      initLoadLibrary();
+      initProperties();
+    }
+    // We *have to* explicitly initialize the streams here, since they're a
+    // blank final field.
+    in = VMSystem.makeStandardInputStream();
+    out = VMSystem.makeStandardOutputStream();
+    err = VMSystem.makeStandardErrorStream();
+    
+    if (! Configuration.JAVA_LANG_SYSTEM_EXPLICIT_INITIALIZATION) {
+      initSystemClassLoader();
+      initSecurityManager();    // Includes getting the class loader.
+    }
+  }
+
+  static void initLoadLibrary () {
     // Note that this loadLibrary() takes precedence over the one in Object,
     // since Object.<clinit> is waiting for System.<clinit> to complete
     // first; but loading a library twice is harmless.
     if (Configuration.INIT_LOAD_LIBRARY)
       loadLibrary("javalang");
 
+  }
+
+  static void initProperties() {
     Properties defaultProperties = Runtime.defaultProperties;
     defaultProperties.put("gnu.classpath.home",
 		    Configuration.CLASSPATH_HOME);
@@ -232,53 +305,40 @@ public final class System
     if (defaultProperties.get("java.io.tmpdir") == null)
       defaultProperties.put("java.io.tmpdir",
                             defaultProperties.get("java.tmpdir"));
+
+    // Note that we use clone here and not new.  Some programs assume
+    // that the system properties do not have a parent.
+    properties = (Properties) Runtime.defaultProperties.clone();
   }
 
-  /**
-   * Stores the current system properties. This can be modified by
-   * {@link #setProperties(Properties)}, but will never be null, because
-   * setProperties(null) sucks in the default properties.
-   */
-  // Note that we use clone here and not new.  Some programs assume
-  // that the system properties do not have a parent.
-  static Properties properties
-    = (Properties) Runtime.defaultProperties.clone();
+  static void initSystemClassLoader() {
+    systemClassLoader = VMClassLoader.getSystemClassLoader();
+  }
 
-  /**
-   * The standard InputStream. This is assigned at startup and starts its
-   * life perfectly valid. Although it is marked final, you can change it
-   * using {@link #setIn(InputStream)} through some hefty VM magic.
-   *
-   * <p>This corresponds to the C stdin and C++ cin variables, which
-   * typically input from the keyboard, but may be used to pipe input from
-   * other processes or files.  That should all be transparent to you,
-   * however.
-   */
-   public static final InputStream in = VMSystem.makeStandardInputStream();
-
-  /**
-   * The standard output PrintStream.  This is assigned at startup and
-   * starts its life perfectly valid. Although it is marked final, you can
-   * change it using {@link #setOut(PrintStream)} through some hefty VM magic.
-   *
-   * <p>This corresponds to the C stdout and C++ cout variables, which
-   * typically output normal messages to the screen, but may be used to pipe
-   * output to other processes or files.  That should all be transparent to
-   * you, however.
-   */
-   public static final PrintStream out = VMSystem.makeStandardOutputStream();
-
-  /**
-   * The standard output PrintStream.  This is assigned at startup and
-   * starts its life perfectly valid. Although it is marked final, you can
-   * change it using {@link #setErr(PrintStream)} through some hefty VM magic.
-   *
-   * <p>This corresponds to the C stderr and C++ cerr variables, which
-   * typically output error messages to the screen, but may be used to pipe
-   * output to other processes or files.  That should all be transparent to
-   * you, however.
-   */
-   public static final PrintStream err = VMSystem.makeStandardErrorStream();
+  static void initSecurityManager () {
+    String secman = properties.getProperty("java.security.manager");
+    if (secman != null)
+      {
+	if (secman.equals("") || secman.equals("default"))
+	  {
+	    Runtime.securityManager = new java.lang.SecurityManager();
+	  }
+	else
+	  {
+	    try
+	      {
+		Class cl = Class.forName(secman, false, systemClassLoader);
+		Runtime.securityManager = (SecurityManager)cl.newInstance();
+	      }
+	    catch (Exception x)
+	      {
+		throw (InternalError)
+		    new InternalError("Unable to create SecurityManager")
+			.initCause(x);
+	      }
+	  }
+      }
+  }
 
   /**
    * This class is uninstantiable.
@@ -353,7 +413,7 @@ public final class System
    * @param sm the new SecurityManager
    * @throws SecurityException if permission is denied
    */
-  public synchronized static void setSecurityManager(SecurityManager sm)
+  public static synchronized void setSecurityManager(SecurityManager sm)
   {
     // Implementation note: the field lives in Runtime because of bootstrap
     // initialization issues. This method is synchronized so that no other
@@ -385,8 +445,9 @@ public final class System
    * @return the current time
    * @see java.util.Date
    */
-  public static long currentTimeMillis() {
-      return VMSystem.currentTimeMillis();
+  public static long currentTimeMillis()
+  {
+    return VMSystem.currentTimeMillis();
   }
 
   /**
@@ -592,6 +653,8 @@ public final class System
    * @throws NullPointerException
    * @throws SecurityException if permission is denied
    * @since 1.5
+   * @specnote This method was deprecated in some JDK releases, but
+   *           was restored in 1.5.
    */
   public static String getenv(String name)
   {
