@@ -39,13 +39,14 @@ exception statement from your version. */
 package java.net;
 
 import gnu.java.net.URLParseError;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
+
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.HashMap;
 import java.util.StringTokenizer;
 
@@ -116,8 +117,8 @@ import java.util.StringTokenizer;
   * done, then the above information is superseded and the behavior of this
   * class in loading protocol handlers is dependent on that factory.
   *
-  * @author Aaron M. Renn <arenn@urbanophile.com>
-  * @author Warren Levy <warrenl@cygnus.com>
+  * @author Aaron M. Renn (arenn@urbanophile.com)
+  * @author Warren Levy (warrenl@cygnus.com)
   *
   * @see URLStreamHandler
   */
@@ -407,6 +408,7 @@ public final class URL implements Serializable
 	    host = context.host;
 	    port = context.port;
 	    file = context.file;
+            userInfo = context.userInfo;
 	    if (file == null || file.length() == 0)
 	      file = "/";
 	    authority = context.authority;
@@ -421,6 +423,7 @@ public final class URL implements Serializable
 	host = context.host;
 	port = context.port;
 	file = context.file;
+        userInfo = context.userInfo;
 	if (file == null || file.length() == 0)
 	  file = "/";
 	authority = context.authority;
@@ -622,6 +625,8 @@ public final class URL implements Serializable
    */
   public String getUserInfo()
   {
+    if (userInfo != null)
+      return userInfo;
     int at = (host == null) ? -1 : host.indexOf('@');
     return at < 0 ? null : host.substring(0, at);
   }
