@@ -1,5 +1,5 @@
 /* DecimalFormat.java -- Formats and parses numbers
-   Copyright (C) 1999, 2000, 2001, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003, 2004  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -62,9 +62,9 @@ public class DecimalFormat extends NumberFormat
 {
   // This is a helper for applyPatternWithSymbols.  It reads a prefix
   // or a suffix.  It can cause some side-effects.
-  private final int scanFix (String pattern, int index, FormatBuffer buf,
-			     String patChars, DecimalFormatSymbols syms,
-			     boolean is_suffix)
+  private int scanFix (String pattern, int index, FormatBuffer buf,
+                       String patChars, DecimalFormatSymbols syms,
+                       boolean is_suffix)
   {
     int len = pattern.length();
     boolean quoteStarted = false;
@@ -140,9 +140,8 @@ public class DecimalFormat extends NumberFormat
   }
 
   // A helper which reads a number format.
-  private final int scanFormat (String pattern, int index,
-				String patChars, DecimalFormatSymbols syms,
-				boolean is_positive)
+  private int scanFormat (String pattern, int index, String patChars,
+                          DecimalFormatSymbols syms, boolean is_positive)
   {
     int max = pattern.length();
 
@@ -294,7 +293,7 @@ public class DecimalFormat extends NumberFormat
 
   // This helper function creates a string consisting of all the
   // characters which can appear in a pattern and must be quoted.
-  private final String patternChars (DecimalFormatSymbols syms)
+  private String patternChars (DecimalFormatSymbols syms)
   {
     StringBuffer buf = new StringBuffer ();
     buf.append(syms.getDecimalSeparator());
@@ -313,8 +312,7 @@ public class DecimalFormat extends NumberFormat
     return buf.toString();
   }
 
-  private final void applyPatternWithSymbols (String pattern,
-					      DecimalFormatSymbols syms)
+  private void applyPatternWithSymbols(String pattern, DecimalFormatSymbols syms)
   {
     // Initialize to the state the parser expects.
     negativePrefix = "";
@@ -425,7 +423,7 @@ public class DecimalFormat extends NumberFormat
     applyPattern (pattern);
   }
 
-  private final boolean equals (String s1, String s2)
+  private boolean equals(String s1, String s2)
   {
     if (s1 == null || s2 == null)
       return s1 == s2;
@@ -847,7 +845,7 @@ public class DecimalFormat extends NumberFormat
     // FIXME: handle Inf and NaN.
 
     // FIXME: do we have to respect minimum digits?
-    // What about leading zeros?  What about multiplier?
+    // What about multiplier?
 
     StringBuffer buf = int_buf;
     StringBuffer frac_buf = null;
@@ -855,7 +853,13 @@ public class DecimalFormat extends NumberFormat
     int start_index = index;
     int max = str.length();
     int exp_index = -1;
-    int last = index + MAXIMUM_INTEGER_DIGITS;
+    int last = index + maximumIntegerDigits; 
+
+    if (maximumFractionDigits > 0)
+      last += maximumFractionDigits + 1;
+    
+    if (useExponentialNotation)
+      last += minExponentDigits + 1;
 
     if (last > 0 && max > last)
       max = last;
@@ -1143,7 +1147,7 @@ public class DecimalFormat extends NumberFormat
     positiveSuffix = newValue;
   }
 
-  private final void quoteFix (StringBuffer buf, String text, String patChars)
+  private void quoteFix(StringBuffer buf, String text, String patChars)
   {
     int len = text.length();
     for (int index = 0; index < len; ++index)
@@ -1160,7 +1164,7 @@ public class DecimalFormat extends NumberFormat
       }
   }
 
-  private final String computePattern (DecimalFormatSymbols syms)
+  private String computePattern(DecimalFormatSymbols syms)
   {
     StringBuffer mainPattern = new StringBuffer ();
     // We have to at least emit a zero for the minimum number of
