@@ -83,8 +83,8 @@ public final class DecimalFormatSymbols implements Cloneable, Serializable
     this (Locale.getDefault());
   }
 
-  private final String safeGetString (ResourceBundle bundle,
-				      String name, String def)
+  private String safeGetString(ResourceBundle bundle,
+                               String name, String def)
   {
     if (bundle != null)
       {
@@ -99,8 +99,8 @@ public final class DecimalFormatSymbols implements Cloneable, Serializable
     return def;
   }
 
-  private final char safeGetChar (ResourceBundle bundle,
-				  String name, char def)
+  private char safeGetChar(ResourceBundle bundle,
+                           String name, char def)
   {
     String r = null;
     if (bundle != null)
