@@ -165,7 +165,7 @@ public class MessageFormat extends Format
       super("");
     }
     
-    private Field(String s)
+    protected Field(String s)
     {
       super(s);
     }
@@ -189,8 +189,7 @@ public class MessageFormat extends Format
   // Helper that returns the text up to the next format opener.  The
   // text is put into BUFFER.  Returns index of character after end of
   // string.  Throws IllegalArgumentException on error.
-  private static final int scanString (String pat, int index,
-				       StringBuffer buffer)
+  private static int scanString(String pat, int index, StringBuffer buffer)
   {
     int max = pat.length();
     buffer.setLength(0);
@@ -220,9 +219,8 @@ public class MessageFormat extends Format
 
   // This helper retrieves a single part of a format element.  Returns
   // the index of the terminating character.
-  private static final int scanFormatElement (String pat, int index,
-					      StringBuffer buffer,
-					      char term)
+  private static int scanFormatElement(String pat, int index,
+                                       StringBuffer buffer, char term)
   {
     int max = pat.length();
     buffer.setLength(0);
@@ -266,9 +264,8 @@ public class MessageFormat extends Format
 
   // This is used to parse a format element and whatever non-format
   // text might trail it.
-  private static final int scanFormat (String pat, int index,
-				       StringBuffer buffer, Vector elts,
-				       Locale locale)
+  private static int scanFormat(String pat, int index, StringBuffer buffer,
+                                Vector elts, Locale locale)
   {
     MessageFormatElement mfe = new MessageFormatElement ();
     elts.addElement(mfe);
