@@ -1,5 +1,5 @@
-/* java.util.zip.ZipFile
-   Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
+/* ZipFile.java --
+   Copyright (C) 2001, 2002, 2003, 2004  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -35,19 +35,19 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package java.util.zip;
 
 import java.io.BufferedInputStream;
 import java.io.DataInput;
-import java.io.File;
-import java.io.InputStream;
-import java.io.IOException;
 import java.io.EOFException;
+import java.io.File;
+import java.io.IOException;
+import java.io.InputStream;
 import java.io.RandomAccessFile;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Iterator;
-import java.util.NoSuchElementException;
 
 /**
  * This class represents a Zip archive.  You can ask for the contained
@@ -148,7 +148,7 @@ public class ZipFile implements ZipConstants
    * @exception IOException if a i/o error occured.
    * @exception EOFException if the file ends prematurely
    */
-  private final int readLeShort(DataInput di, byte[] b) throws IOException
+  private int readLeShort(DataInput di, byte[] b) throws IOException
   {
     di.readFully(b, 0, 2);
     return (b[0] & 0xff) | (b[1] & 0xff) << 8;
@@ -165,14 +165,13 @@ public class ZipFile implements ZipConstants
    * @exception IOException if a i/o error occured.
    * @exception EOFException if the file ends prematurely
    */
-  private final int readLeInt(DataInput di, byte[] b) throws IOException
+  private int readLeInt(DataInput di, byte[] b) throws IOException
   {
     di.readFully(b, 0, 4);
     return ((b[0] & 0xff) | (b[1] & 0xff) << 8)
 	    | ((b[2] & 0xff) | (b[3] & 0xff) << 8) << 16;
   }
 
-  
   /**
    * Read an unsigned short in little endian byte order from the given
    * byte buffer at the given offset.
@@ -181,7 +180,7 @@ public class ZipFile implements ZipConstants
    * @param off the offset to read from.
    * @return The value read.
    */
-  private final int readLeShort(byte[] b, int off)
+  private int readLeShort(byte[] b, int off)
   {
     return (b[off] & 0xff) | (b[off+1] & 0xff) << 8;
   }
@@ -194,7 +193,7 @@ public class ZipFile implements ZipConstants
    * @param off the offset to read from.
    * @return The value read.
    */
-  private final int readLeInt(byte[] b, int off)
+  private int readLeInt(byte[] b, int off)
   {
     return ((b[off] & 0xff) | (b[off+1] & 0xff) << 8)
 	    | ((b[off+2] & 0xff) | (b[off+3] & 0xff) << 8) << 16;
