@@ -37,19 +37,15 @@ exception statement from your version. */
 
 package javax.swing;
 
-import java.awt.Color;
 import java.awt.Component;
 import java.awt.Container;
 import java.awt.Dimension;
-import java.awt.Graphics;
 import java.awt.Insets;
 import java.awt.Rectangle;
 import java.beans.PropertyVetoException;
 import java.io.Serializable;
-import javax.swing.JDesktopPane;
-import javax.swing.JInternalFrame;
-import javax.swing.JInternalFrame.JDesktopIcon;
 
+import javax.swing.JInternalFrame.JDesktopIcon;
 
 /**
  * DefaultDesktopManager is the default implementation of DesktopManager for
@@ -84,7 +80,7 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
   private transient Container pane;
 
   /**
-   * An array of Rectangles that holds the bounds of the JDesktopIcons in the 
+   * An array of Rectangles that holds the bounds of the JDesktopIcons in the
    * JDesktopPane when looking for where to place a new icon.
    */
   private transient Rectangle[] iconRects;
@@ -436,7 +432,6 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
                           int newWidth, int newHeight)
   {
     dragCache.setBounds(newX, newY, newWidth, newHeight);
-    dragCache = findMinimum(dragCache, component);
 
     if (currentDragMode == JDesktopPane.OUTLINE_DRAG_MODE)
       {
@@ -628,27 +623,4 @@ public class DefaultDesktopManager implements DesktopManager, Serializable
   {
     return frame.getWasIcon();
   } // wasIcon()
-
-  /**
-   * This is a helper method that determines the minimum size a 
-   * JInternalFrame can be resized to.
-   *
-   * @param r The desired size.
-   * @param c The JComponent to find a minimum size for.
-   *
-   * @return The minimum size a JInternalFrame can be resized to.
-   */
-  private Rectangle findMinimum(Rectangle r, JComponent c)
-  {
-    if (r != null && c != null)
-      {
-	Dimension d = c.getPreferredSize();
-	if (d != null)
-	  {
-	    r.width = Math.max(d.width, r.width);
-	    r.height = Math.max(d.height, r.height);
-	  }
-      }
-    return r;
-  }
 } // DefaultDesktopManager
