@@ -35,22 +35,20 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package javax.swing;
 
 import java.awt.Component;
 import java.awt.Font;
 import java.awt.Image;
 import java.awt.event.KeyEvent;
+
 import javax.accessibility.Accessible;
 import javax.accessibility.AccessibleContext;
-import javax.swing.Icon;
 import javax.swing.plaf.LabelUI;
 
-
 /**
- * <p>
  * A swing widget that displays a text message and/or an icon.
- * </p>
  */
 public class JLabel extends JComponent implements Accessible, SwingConstants
 {
@@ -368,7 +366,7 @@ public class JLabel extends JComponent implements Accessible, SwingConstants
   public void setDisplayedMnemonic(int mnemonic)
   {
     if (displayedMnemonic != mnemonic)
-  {
+      {
 	firePropertyChange(DISPLAYED_MNEMONIC_CHANGED_PROPERTY,
 	                   displayedMnemonic, mnemonic);
 	displayedMnemonic = mnemonic;
@@ -386,7 +384,7 @@ public class JLabel extends JComponent implements Accessible, SwingConstants
    * @param mnemonic The character to use for the mnemonic.
    */
   public void setDisplayedMnemonic(char mnemonic)
-      {
+  {
     setDisplayedMnemonic((int) mnemonic);
   }
 
@@ -416,16 +414,16 @@ public class JLabel extends JComponent implements Accessible, SwingConstants
   {
     if (newIndex < -1 || (text != null && newIndex >= text.length()))
       throw new IllegalArgumentException();
-      
+
     if (text == null || text.charAt(newIndex) != displayedMnemonic)
       newIndex = -1;
-      
+
     if (newIndex != displayedMnemonicIndex)
-    {
-      firePropertyChange(DISPLAYED_MNEMONIC_INDEX_CHANGED_PROPERTY,
-                           displayedMnemonicIndex, newIndex);
-        displayedMnemonicIndex = newIndex;
-    }
+      {
+	firePropertyChange(DISPLAYED_MNEMONIC_INDEX_CHANGED_PROPERTY,
+	                   displayedMnemonicIndex, newIndex);
+	displayedMnemonicIndex = newIndex;
+      }
   }
 
   /**
