@@ -1,4 +1,4 @@
-/* BasicSliderUI.java
+/* BasicSliderUI.java --
    Copyright (C) 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
@@ -35,6 +35,7 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package javax.swing.plaf.basic;
 
 import java.awt.Color;
@@ -73,7 +74,6 @@ import javax.swing.event.MouseInputAdapter;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.SliderUI;
 
-
 /**
  * <p>
  * BasicSliderUI.java This is the UI delegate in the Basic look and feel that
@@ -152,7 +152,6 @@ public class BasicSliderUI extends SliderUI
     {
       // Maximum, minimum, and extent values will be taken
       // care of automatically when the slider is repainted.
-      
       // Only thing that needs recalculation is the thumb.
       calculateThumbLocation();
       slider.repaint();
@@ -226,18 +225,18 @@ public class BasicSliderUI extends SliderUI
       if (e.getPropertyName().equals(JSlider.ORIENTATION_CHANGED_PROPERTY))
 	recalculateIfOrientationChanged();
       else if (e.getPropertyName().equals(JSlider.MODEL_CHANGED_PROPERTY))
-      {
-        BoundedRangeModel oldModel = (BoundedRangeModel) e.getOldValue();
-	oldModel.removeChangeListener(changeListener);
-	slider.getModel().addChangeListener(changeListener);
-	calculateThumbLocation();
-      }
+        {
+	  BoundedRangeModel oldModel = (BoundedRangeModel) e.getOldValue();
+	  oldModel.removeChangeListener(changeListener);
+	  slider.getModel().addChangeListener(changeListener);
+	  calculateThumbLocation();
+        }
+
       // elif the componentOrientation changes (this is a bound property,
       // just undocumented) we change leftToRightCache. In Sun's 
       // implementation, the LTR cache changes on a repaint. This is strange
       // since there is no need to do so. We could events here and 
       // update the cache. 
-      
       // elif the border/insets change, we recalculateInsets.
       slider.repaint();
     }
@@ -322,7 +321,7 @@ public class BasicSliderUI extends SliderUI
   /**
    * Helper class that listens for mouse events.
    */
-  protected class TrackListener extends MouseInputAdapter
+  public class TrackListener extends MouseInputAdapter
   {
     /** The current X position of the mouse. */
     protected int currentMouseX;
@@ -330,8 +329,9 @@ public class BasicSliderUI extends SliderUI
     /** The current Y position of the mouse. */
     protected int currentMouseY;
 
-    /** The offset between the current slider value
-        and the cursor's position. */
+    /**
+     * The offset between the current slider value and the cursor's position.
+     */
     protected int offset;
 
     /**
@@ -346,15 +346,15 @@ public class BasicSliderUI extends SliderUI
       currentMouseX = e.getX();
       currentMouseY = e.getY();
       if (slider.getValueIsAdjusting())
-      {
-        int value;
-        if (slider.getOrientation() == JSlider.HORIZONTAL)
-	  value = valueForXPosition(currentMouseX) - offset;
-	else
-	  value = valueForYPosition(currentMouseY) - offset;
-	
-	slider.setValue(value);
-      }
+        {
+	  int value;
+	  if (slider.getOrientation() == JSlider.HORIZONTAL)
+	    value = valueForXPosition(currentMouseX) - offset;
+	  else
+	    value = valueForYPosition(currentMouseY) - offset;
+
+	  slider.setValue(value);
+        }
     }
 
     /**
@@ -390,11 +390,8 @@ public class BasicSliderUI extends SliderUI
       if (slider.getSnapToTicks())
 	value = findClosestTick(value);
 
-      if (value == slider.getValue())
-	return;
-
       // If the thumb is hit, then we don't need to set the timers to move it. 
-      if (!thumbRect.contains(e.getPoint()))
+      if (! thumbRect.contains(e.getPoint()))
         {
 	  // The mouse has hit some other part of the slider.
 	  // The value moves no matter where in the slider you hit.
@@ -406,8 +403,8 @@ public class BasicSliderUI extends SliderUI
       else
         {
 	  slider.setValueIsAdjusting(true);
-          offset = value - slider.getValue();
-	}
+	  offset = value - slider.getValue();
+        }
     }
 
     /**
@@ -422,11 +419,11 @@ public class BasicSliderUI extends SliderUI
       currentMouseY = e.getY();
 
       if (slider.getValueIsAdjusting())
-      {
-        slider.setValueIsAdjusting(false);
-	if (slider.getSnapToTicks())
-	  slider.setValue(findClosestTick(slider.getValue()));
-      }
+        {
+	  slider.setValueIsAdjusting(false);
+	  if (slider.getSnapToTicks())
+	    slider.setValue(findClosestTick(slider.getValue()));
+        }
       if (scrollTimer != null)
 	scrollTimer.stop();
     }
@@ -857,83 +854,83 @@ public class BasicSliderUI extends SliderUI
    */
 
   /**
-   * This method returns the preferred size when the slider is
-   * horizontally oriented.
+   * This method returns the preferred size when the slider is horizontally
+   * oriented.
    *
    * @return The dimensions of the preferred horizontal size.
    */
   public Dimension getPreferredHorizontalSize()
   {
     Insets insets = slider.getInsets();
-    
+
     // The width should cover all the labels (which are usually the
     // deciding factor of the width)
-    int width = getWidthOfWidestLabel() * (slider.getLabelTable() == null ? 
-                                           0 : slider.getLabelTable().size());
-    
+    int width = getWidthOfWidestLabel() * (slider.getLabelTable() == null ? 0
+                                                                          : slider.getLabelTable()
+                                                                                  .size());
+
     // If there are not enough labels.
     // This number is pretty much arbitrary, but it looks nice.
     if (width < 200)
       width = 200;
-    
+
     // We can only draw inside of the focusRectangle, so we have to
     // pad it with insets.
-    width += insets.left + insets.right + focusInsets.left + 
-             focusInsets.right;
-      
+    width += insets.left + insets.right + focusInsets.left + focusInsets.right;
+
     // Height is determined by the thumb, the ticks and the labels.
     int height = thumbHeight;
 
-    if (slider.getPaintTicks() && slider.getMajorTickSpacing() > 0 ||
-        slider.getMinorTickSpacing() > 0)
+    if (slider.getPaintTicks() && slider.getMajorTickSpacing() > 0
+        || slider.getMinorTickSpacing() > 0)
       height += tickHeight;
 
     if (slider.getPaintLabels())
       height += getHeightOfTallestLabel();
-    
-    height += insets.top + insets.bottom + focusInsets.top + 
-              focusInsets.bottom;
-	      
+
+    height += insets.top + insets.bottom + focusInsets.top
+    + focusInsets.bottom;
+
     return new Dimension(width, height);
   }
 
   /**
-   * This method returns the preferred size when the slider is
-   * vertically oriented.
+   * This method returns the preferred size when the slider is vertically
+   * oriented.
    *
    * @return The dimensions of the preferred vertical size.
    */
   public Dimension getPreferredVerticalSize()
   {
     Insets insets = slider.getInsets();
-    
-    int height = getHeightOfTallestLabel() * (slider.getLabelTable() == null ? 
-                                              0 : slider.getLabelTable().size());
-    
+
+    int height = getHeightOfTallestLabel() * (slider.getLabelTable() == null
+                                              ? 0 : slider.getLabelTable()
+                                                          .size());
+
     if (height < 200)
       height = 200;
-      
-    height += insets.top + insets.bottom + focusInsets.top + 
-             focusInsets.bottom;
+
+    height += insets.top + insets.bottom + focusInsets.top
+    + focusInsets.bottom;
 
     int width = thumbHeight;
-    
-    if (slider.getPaintTicks() && slider.getMajorTickSpacing() > 0 ||
-        slider.getMinorTickSpacing() > 0)
+
+    if (slider.getPaintTicks() && slider.getMajorTickSpacing() > 0
+        || slider.getMinorTickSpacing() > 0)
       width += tickHeight;
 
     if (slider.getPaintLabels())
       width += getWidthOfWidestLabel();
 
-    width += insets.left + insets.right + focusInsets.left + 
-             focusInsets.right;
-	     
+    width += insets.left + insets.right + focusInsets.left + focusInsets.right;
+
     return new Dimension(width, height);
   }
 
   /**
-   * This method returns the minimum size when the slider is
-   * horizontally oriented.
+   * This method returns the minimum size when the slider is horizontally
+   * oriented.
    *
    * @return The dimensions of the minimum horizontal size.
    */
@@ -943,8 +940,8 @@ public class BasicSliderUI extends SliderUI
   }
 
   /**
-   * This method returns the minimum size of the slider when it 
-   * is vertically oriented.
+   * This method returns the minimum size of the slider when it  is vertically
+   * oriented.
    *
    * @return The dimensions of the minimum vertical size.
    */
@@ -963,7 +960,7 @@ public class BasicSliderUI extends SliderUI
    * @return The dimensions of the preferred size.
    */
   public Dimension getPreferredSize(JComponent c)
-  {   
+  {
     if (slider.getOrientation() == JSlider.HORIZONTAL)
       return getPreferredHorizontalSize();
     else
@@ -1003,22 +1000,22 @@ public class BasicSliderUI extends SliderUI
     else
       return getPreferredVerticalSize();
   }
-  
+
   /**
-   * This method calculates all the sizes of the rectangles by delegating
-   * to the helper methods calculateXXXRect.
+   * This method calculates all the sizes of the rectangles by delegating to
+   * the helper methods calculateXXXRect.
    */
-   protected void calculateGeometry()
-   {
-     calculateFocusRect();
-     calculateContentRect();
-     calculateThumbSize();
-     calculateTrackBuffer();
-     calculateTrackRect();
-     calculateTickRect();
-     calculateLabelRect();
-     calculateThumbLocation();
-   }
+  protected void calculateGeometry()
+  {
+    calculateFocusRect();
+    calculateContentRect();
+    calculateThumbSize();
+    calculateTrackBuffer();
+    calculateTrackRect();
+    calculateTickRect();
+    calculateLabelRect();
+    calculateThumbLocation();
+  }
 
   /**
    * This method calculates the size and position of the focusRect. This
@@ -1230,6 +1227,7 @@ public class BasicSliderUI extends SliderUI
     if (slider.getLabelTable() == null)
       return 0;
 
+    Dimension pref;
     for (Enumeration list = slider.getLabelTable().elements();
          list.hasMoreElements();)
       {
@@ -1237,8 +1235,9 @@ public class BasicSliderUI extends SliderUI
 	if (! (comp instanceof Component))
 	  continue;
 	label = (Component) comp;
-	if (label.getWidth() > widest)
-	  widest = label.getWidth();
+	pref = label.getPreferredSize();
+	if (pref != null && pref.width > widest)
+	  widest = pref.width;
       }
     return widest;
   }
@@ -1256,7 +1255,7 @@ public class BasicSliderUI extends SliderUI
 
     if (slider.getLabelTable() == null)
       return 0;
-
+    Dimension pref;
     for (Enumeration list = slider.getLabelTable().elements();
          list.hasMoreElements();)
       {
@@ -1264,8 +1263,9 @@ public class BasicSliderUI extends SliderUI
 	if (! (comp instanceof Component))
 	  continue;
 	label = (Component) comp;
-	if (label.getHeight() > tallest)
-	  tallest = label.getHeight();
+	pref = label.getPreferredSize();
+	if (pref != null && pref.height > tallest)
+	  tallest = pref.height;
       }
     return tallest;
   }
@@ -1412,7 +1412,7 @@ public class BasicSliderUI extends SliderUI
     leftToRightCache = slider.getComponentOrientation() != ComponentOrientation.RIGHT_TO_LEFT;
     // FIXME: This next line is only here because the above line is here.
     calculateThumbLocation();
-    
+
     if (slider.getPaintTrack())
       paintTrack(g);
     if (slider.getPaintTicks())
@@ -1539,18 +1539,17 @@ public class BasicSliderUI extends SliderUI
 	c.translate((trackRect.width / 2) + (width / 2), trackRect.height);
 	d.translate((trackRect.width / 2) + (width / 2), 0);
       }
-    high = new Polygon(new int[] { b.x, c.x, d.x },
-                       new int[] { b.y, c.y, d.y }, 3);
-    shadow = new Polygon(new int[] { b.x, a.x, d.x },
-                         new int[] { b.y, a.y, d.y }, 3);
+    g.setColor(Color.GRAY);
+    g.fillRect(a.x, a.y, width, height);
 
     g.setColor(getHighlightColor());
-    g.drawPolygon(high);
+    g.drawLine(b.x, b.y, c.x, c.y);
+    g.drawLine(c.x, c.y, d.x, d.y);
+
     g.setColor(getShadowColor());
-    g.drawPolygon(shadow);
+    g.drawLine(b.x, b.y, a.x, a.y);
+    g.drawLine(a.x, a.y, d.x, d.y);
 
-    g.setColor(Color.GRAY);
-    g.fillRect(a.x + 1, a.y + 1, width - 2, height - 2);
     g.setColor(saved_color);
   }
 
@@ -1574,8 +1573,8 @@ public class BasicSliderUI extends SliderUI
 	  {
 	    double loc = tickRect.x;
 	    double increment = (max == min) ? 0
-	                                 : majorSpace * (double) tickRect.width / (max
-	                                 - min);
+	                                    : majorSpace * (double) tickRect.width / (max
+	                                    - min);
 	    if (drawInverted())
 	      {
 		loc += tickRect.width;
@@ -1591,8 +1590,8 @@ public class BasicSliderUI extends SliderUI
 	  {
 	    double loc = tickRect.height + tickRect.y;
 	    double increment = (max == min) ? 0
-	                                 : -majorSpace * (double) tickRect.height / (max
-	                                 - min);
+	                                    : -majorSpace * (double) tickRect.height / (max
+	                                    - min);
 	    if (drawInverted())
 	      {
 		loc = tickRect.y;
@@ -1611,8 +1610,8 @@ public class BasicSliderUI extends SliderUI
 	  {
 	    double loc = tickRect.x;
 	    double increment = (max == min) ? 0
-	                                 : minorSpace * (double) tickRect.width / (max
-	                                 - min);
+	                                    : minorSpace * (double) tickRect.width / (max
+	                                    - min);
 	    if (drawInverted())
 	      {
 		loc += tickRect.width;
@@ -1628,8 +1627,8 @@ public class BasicSliderUI extends SliderUI
 	  {
 	    double loc = tickRect.height + tickRect.y;
 	    double increment = (max == min) ? 0
-	                                 : -minorSpace * (double) tickRect.height / (max
-	                                 - min);
+	                                    : -minorSpace * (double) tickRect.height / (max
+	                                    - min);
 	    if (drawInverted())
 	      {
 		loc = tickRect.y;
@@ -1662,8 +1661,11 @@ public class BasicSliderUI extends SliderUI
                                               Rectangle tickBounds, int x)
   {
     int y = tickRect.y + tickRect.height / 4;
+    Color saved = g.getColor();
+    g.setColor(Color.BLACK);
 
     g.drawLine(x, y, x, y + tickRect.height / 4);
+    g.setColor(saved);
   }
 
   /**
@@ -1678,8 +1680,11 @@ public class BasicSliderUI extends SliderUI
                                               Rectangle tickBounds, int x)
   {
     int y = tickRect.y + tickRect.height / 4;
+    Color saved = g.getColor();
+    g.setColor(Color.BLACK);
 
     g.drawLine(x, y, x, y + tickRect.height / 2);
+    g.setColor(saved);
   }
 
   /**
@@ -1694,8 +1699,11 @@ public class BasicSliderUI extends SliderUI
                                              int y)
   {
     int x = tickRect.x + tickRect.width / 4;
+    Color saved = g.getColor();
+    g.setColor(Color.BLACK);
 
     g.drawLine(x, y, x + tickRect.width / 4, y);
+    g.setColor(saved);
   }
 
   /**
@@ -1710,8 +1718,11 @@ public class BasicSliderUI extends SliderUI
                                              int y)
   {
     int x = tickRect.x + tickRect.width / 4;
+    Color saved = g.getColor();
+    g.setColor(Color.BLACK);
 
     g.drawLine(x, y, x + tickRect.width / 2, y);
+    g.setColor(saved);
   }
 
   /**
@@ -1786,17 +1797,16 @@ public class BasicSliderUI extends SliderUI
     // the labels may not fit inside the slider's bounds. Rather than mucking 
     // with font sizes and possible icon sizes, we'll set the bounds for
     // the label and let it get clipped.
-
     Dimension dim = label.getPreferredSize();
     int w = (int) dim.getWidth();
     int h = (int) dim.getHeight();
-    
+
     int max = slider.getMaximum();
     int min = slider.getMinimum();
 
     if (value > max || value < min)
       return;
-    
+
     //           value
     //             |
     //        ------------
@@ -1804,8 +1814,6 @@ public class BasicSliderUI extends SliderUI
     //        |          |
     //        |          |
     //  The label must move w/2 to the right to fit directly under the value.
-    
-    
     int xpos = xPositionForValue(value) - w / 2;
     int ypos = labelRect.y;
 
@@ -1948,10 +1956,10 @@ public class BasicSliderUI extends SliderUI
       }
 
     g.setColor(Color.WHITE);
-    g.drawPolygon(bright);
+    g.drawPolyline(bright.xpoints, bright.ypoints, bright.npoints);
 
     g.setColor(Color.BLACK);
-    g.drawPolygon(dark);
+    g.drawPolyline(dark.xpoints, dark.ypoints, dark.npoints);
 
     g.setColor(Color.GRAY);
     g.fillPolygon(all);
@@ -2019,7 +2027,7 @@ public class BasicSliderUI extends SliderUI
   protected void scrollDueToClickInTrack(int dir)
   {
     scrollTimer.stop();
-  
+
     scrollListener.setDirection(dir);
     scrollListener.setScrollByBlock(true);
 
