@@ -1,5 +1,5 @@
 /* gtkpeer.h -- Some global variables and #defines
-   Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -107,11 +107,17 @@ extern struct state_table *native_pixbufdecoder_state_table;
 
 #endif /* JVM_SUN */
 
+#define SWAPU32(w)							\
+  (((w) << 24) | (((w) & 0xff00) << 8) | (((w) >> 8) & 0xff00) | ((w) >> 24))
+
 struct graphics
 {
   GdkDrawable *drawable;
   GdkGC *gc;
   GdkColormap *cm;
+  PangoFontDescription *pango_font;
+  PangoContext *pango_context;
+  PangoLayout *pango_layout;
   jint x_offset, y_offset;
 };
 
@@ -444,6 +450,13 @@ extern jmethodID postListItemEventID;
 extern jmethodID postTextEventID;
 extern jmethodID postWindowEventID;
 
+extern jmethodID beginNativeRepaintID;
+extern jmethodID endNativeRepaintID;
+
+extern jmethodID initComponentGraphicsID;
+extern jmethodID initComponentGraphics2DID;
+extern jmethodID setCursorID;
+
 extern jmethodID syncAttrsID;
 extern jclass gdkColor;
 extern jmethodID gdkColorID;
@@ -460,10 +473,10 @@ gboolean pre_event_handler (GtkWidget *widget,
 			       jobject peer);
 
 void connect_awt_hook (JNIEnv *env, jobject peer_obj, int nwindows, ...);
+void connect_awt_hook_cb (GtkWidget *widget, jobject peer);
 
 void set_visible (GtkWidget *widget, jboolean visible);
 void set_parent (GtkWidget *widget, GtkContainer *parent);
-GtkLayout *find_gtk_layout (GtkWidget *parent);
 
 jint keyevent_state_to_awt_mods (GdkEvent *event);
 
@@ -473,6 +486,13 @@ struct item_event_hook_info
   const char *label;
 };
 
+/* Union used for type punning. */
+union widget_union
+{
+  void **void_widget;
+  GtkWidget **widget;
+};
+
 #define DEBUG_LOCKING 0
 
 #if DEBUG_LOCKING
