@@ -1,5 +1,5 @@
-/* generic_math_int64.h - Native methods for 64bit math operations
-   Copyright (C) 1998 Free Software Foundation, Inc.
+/* target_generic_file - Native methods for file operations
+   Copyright (C) 1998, 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -413,14 +413,14 @@ extern "C" {
 \***********************************************************************/
 
 #ifndef TARGET_NATIVE_FILE_AVAILABLE
-  #if   defined(HAVE_FIONREAD)
-    #ifdef HAVE_SYS_IOCTL
-      #define BSD_COMP /* Get FIONREAD on Solaris2 */
-      #include <sys/ioctl.h>
-    #endif
-    #ifdef HAVE_SYS_FILIO /* Get FIONREAD on Solaris 2.5 */
-      #include <sys/filio.h>
-    #endif
+  #ifdef HAVE_SYS_IOCTL_H
+    #define BSD_COMP /* Get FIONREAD on Solaris2 */
+    #include <sys/ioctl.h>
+  #endif
+  #ifdef HAVE_SYS_FILIO_H /* Get FIONREAD on Solaris 2.5 */
+    #include <sys/filio.h>
+  #endif
+  #if defined (FIONREAD)
     #define TARGET_NATIVE_FILE_AVAILABLE(filedescriptor,length,result) \
       do { \
         ssize_t __n; \
@@ -795,7 +795,7 @@ extern "C" {
 * Notes      : -
 \***********************************************************************/
 
-//??? name als buffer?
+/* XXX ??? name als buffer? */
 #ifndef TARGET_NATIVE_FILE_READ_DIR
   #include <sys/types.h>
   #include <dirent.h>
